package com.sun.org.apache.bcel.internal.generic;

import java.util.ArrayList;
import com.sun.org.apache.bcel.internal.classfile.Attribute;
import java.util.List;
import com.sun.org.apache.bcel.internal.classfile.AccessFlags;

public abstract class FieldGenOrMethodGen extends AccessFlags implements NamedAndTyped, Cloneable
{
    private String name;
    private Type type;
    private ConstantPoolGen cp;
    private final List<Attribute> attribute_vec;
    private final List<AnnotationEntryGen> annotation_vec;
    
    protected FieldGenOrMethodGen() {
        this.attribute_vec = new ArrayList();
        this.annotation_vec = new ArrayList();
    }
    
    protected FieldGenOrMethodGen(int access_flags) {
        super(access_flags);
        this.attribute_vec = new ArrayList();
        this.annotation_vec = new ArrayList();
    }
    
    @Override
    public void setType(Type type) {
        if (type.getType() == 16) {
            throw new IllegalArgumentException("Type can not be " + type);
        }
        this.type = type;
        return;
    }
    
    @Override
    public Type getType() {
        return this.type;
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    @Override
    public void setName(String name) {
        this.name = name;
    }
    
    public ConstantPoolGen getConstantPool() {
        return this.cp;
    }
    
    public void setConstantPool(ConstantPoolGen cp) {
        this.cp = cp;
    }
    
    public void addAttribute(Attribute a) {
        this.attribute_vec.add(a);
    }
    
    protected void addAnnotationEntry(AnnotationEntryGen ag) {
        this.annotation_vec.add(ag);
    }
    
    public void removeAttribute(Attribute a) {
        this.attribute_vec.remove(a);
    }
    
    protected void removeAnnotationEntry(AnnotationEntryGen ag) {
        this.annotation_vec.remove(ag);
    }
    
    public void removeAttributes() {
        this.attribute_vec.clear();
    }
    
    protected void removeAnnotationEntries() {
        this.annotation_vec.clear();
    }
    
    public Attribute[] getAttributes() {
        Attribute[] attributes = new Attribute[this.attribute_vec.size()];
        this.attribute_vec.toArray(attributes);
        return attributes;
    }
    
    public AnnotationEntryGen[] getAnnotationEntries() {
        AnnotationEntryGen[] annotations = new AnnotationEntryGen[this.annotation_vec.size()];
        this.annotation_vec.toArray(annotations);
        return annotations;
    }
    
    public abstract String getSignature();
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException e) {
            throw new Error("Clone Not Supported");
        }
    }
}
